==> ondir-0.1.2.tar.gz: ondir-0.1.2/src/main.cc <==
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <climits>
#include <cstdio>
#include <iostream>
#include <string>
#include <vector>
#include "config.h"
#include "String.h"
#include "RCParser.h"
#include "main.h"

using namespace std;

enum Shell {
	BASH,
	TCSH
};

int main(int argc, char **argv) {
bool verbose = false;
Shell shell = BASH;
char *olddir = argv[1];
RCParser rc;

	try {
		rc("/etc/ondirrc");
		rc("/home/athomas/.ondirrc");
	} catch (Lexer::exception &e) {
		cerr << "error:" << e.line() << ": " << e.what() << endl;
		return 1;
	}

	if (getuid() == 0) {
		cerr << argv[0] << ": error, refusing to run as root" << endl;
		return 1;
	}
		
	// parse options
	{
	int i;
		for (i = 1; i < argc && argv[i][0] == '-'; ++i)
			switch (argv[i][1]) {
				case 'V' :
					cout << VERSION << endl;
					return 0;
				break;
				case 'v' : // verbose
					verbose = true;
				break;
				case 't' : // tcsh
					shell = TCSH;
				break;
				default :
					cerr << argv[0] << ": error, unknown option '" << argv[i] << "'" << endl;
				case 'h' :
					cout << "usage: " << argv[0] << " [-v] [-t] [-h] <old directory>" << endl
						<< "  -v    verbose" << endl
						<< "  -t    display TCSH commands rather than BASH" << endl
						<< "  -h    help" << endl
						<< "  -V    show version" << endl;
					return 0;
				break;
			}
		if (i >= argc) {
			cerr << argv[0] << ": error, path must be specified" << endl;
			return 1;
		}
		olddir = argv[i];
	}

	// get base working directories
vector<string> last_dir, current_dir;

	{
	char tmp[strlen(olddir) + 1];
		strcpy(tmp, olddir);
		if (tmp[0] != '/') {
			cerr << argv[0] << ": error, path must be absolute" << endl;
			return 1;
		}
		while (tmp[strlen(tmp) - 1] == '/') tmp[strlen(tmp) - 1] = 0;
		last_dir = split(tmp);
	}
	{
	char tmp[PATH_MAX];
		getcwd(tmp, PATH_MAX);
		while (tmp[strlen(tmp) - 1] == '/') tmp[strlen(tmp) - 1] = 0;
		current_dir = split(tmp);
	}

unsigned common_root;

	// Find common root path
	for (common_root = 0; last_dir.size() > common_root && current_dir.size() > common_root && last_dir[common_root] == current_dir[common_root] ; common_root++) ;
	if (common_root) common_root--;

	// Move up to common root from old path
	for (unsigned i = last_dir.size() - 1; i > common_root; i--) {
	string leaving = join(last_dir, i);
	string filename = leaving + "/.onleave";


		if (!readable(filename))
			if (readable(leaving + "/.onexit"))
				filename = leaving + "/.onexit";

		if (readable(filename)) {
			if (!goodpermissions(leaving + "/.onleave"))
				cerr << argv[0] << ": warning, not executing \"" << leaving << "/.onleave\" as it has bad permissions/ownership" << endl;
			else {
				if (verbose) cerr << "ondir: called .onleave in '" << leaving << "'" << endl;
				if (shell == TCSH) cout << "setenv ";
				cout << "ONDIRWD=\"" << leaving << "\"" << endl;
				if (shell == BASH)
					cout << ".";
				if (shell == TCSH)
					cout << "source";
				cout << " \"" << leaving << "/.onleave\"" << endl;
			}
			if (shell == TCSH) 
				cout << endl << "unsetenv ONDIRWD" << endl;
			else
				cout << endl << "unset ONDIRWD" << endl;
		} else 
		if (rc.leavePath(leaving) != "") {
		string code = rc.leavePath(leaving);

			if (verbose) cerr << "ondir: executing leave code for '" << leaving << "'" << endl;
			if (shell == TCSH) cout << "setenv ";
			cout << "ONDIRWD=\"" << leaving << "\";";
			cout << str::replace("\n", ";", code) << ";";
			if (shell == TCSH)
				cout << endl << "unsetenv ONDIRWD" << endl;
			else
				cout << endl << "unset ONDIRWD" << endl;
		}
	}
	// Move in to new path from common root
	for (unsigned i = common_root + 1; i < current_dir.size(); i++) {
	string entering = join(current_dir, i);

		if (readable(entering + "/.onenter")) {
			if (!goodpermissions(entering + "/.onenter"))
				cerr << argv[0] << ": warning, not executing \"" << entering << "/.onenter\" as it has bad permissions/ownership" << endl;
			else {
				if (verbose) cerr << "ondir: called .onenter in '" << entering << "'" << endl;
				if (shell == TCSH) cout << "setenv ";
				cout << "ONDIRWD=\"" << entering << "\"" << endl;
				if (shell == BASH)
					cout << ".";
				if (shell == TCSH)
					cout << "source";
				cout << " \"" << entering << "/.onenter\"" << endl;
			}
			if (shell == TCSH) 
				cout << endl << "unsetenv ONDIRWD" << endl;
			else
				cout << endl << "unset ONDIRWD" << endl;
		} else 
		if (rc.enterPath(entering) != "") {
		string code = rc.enterPath(entering);

			if (verbose) cerr << "ondir: executing enter code for '" << entering << "'" << endl;
			if (shell == TCSH) cout << "setenv ";
			cout << "ONDIRWD=\"" << entering << "\";";
			cout << str::replace("\n", ";", code) << ";";
			if (shell == TCSH) 
				cout << endl << "unsetenv ONDIRWD" << endl;
			else
				cout << endl << "unset ONDIRWD" << endl;
		}
	}
	return 0;
}

/* Split a path into a vector */
vector<string> split(char const *str) {
vector<string> out;
char const *last;

	if (str[0] == '/') str++;
	for (last = str; *str; str++) {
		if (*str == '/') {
			out.push_back(string(last, str - last));
			last = str + 1;
		}
	}
	out.push_back(string(last, str - last));
	return out;
}

/* Join a split path into a string */
string join(vector<string> const &in, unsigned to) {
string out;

	for (unsigned i = 0; i <= to; ++i)
		out += "/" + in[i];
	return out;
}

/* Return true if file is readable */
bool readable(string const &file) {
struct stat s;

	if (!stat(file.c_str(), &s) && 
		s.st_mode & (S_IRUSR | S_IRGRP | S_IROTH))
			return true;
	return false;
}

/* Return true if file has good permissions */
bool goodpermissions(string const &file) {
struct stat s;

	if (!stat(file.c_str(), &s) && s.st_uid == getuid() &&
		!(s.st_mode & (S_IWGRP | S_IWOTH)))
			return true;
	return false;
}

void enterPath(string const &path) {
}

void exitPath(string const &path) {
}

string expandString(string const &str) {
string out;

	for (char const *i = str.c_str(); *i != 0; ++i)
		if (*i == '$') {
		string word;

			++i;
			while (isalnum(*i) || *i == '_') {
				word += *i;
				++i;
			}
			--i;
			if (getenv(word.c_str()))
				out += getenv(word.c_str());
		} else
			out += *i;
	return out;
}

==> ondir-0.1.3.tar.gz: ondir-0.1.3/src/main.cc <==
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <climits>
#include <cstdio>
#include <iostream>
#include <string>
#include <vector>
#include "config.h"
#include "String.h"
#include "RCParser.h"
#include "main.h"

using namespace std;

enum Shell {
	BASH,
	TCSH
};

int main(int argc, char **argv) {
bool verbose = false;
Shell shell = BASH;
char *olddir = argv[1];
RCParser rc;

	try {
		rc("/etc/ondirrc");
		if (getenv("HOME")) {
		string home = string(getenv("HOME")) + "/.ondirrc";

			rc(home);
		}
	} catch (Lexer::exception &e) {
		cerr << "error:" << e.line() << ": " << e.what() << endl;
		return 1;
	}

	if (getuid() == 0) {
		cerr << argv[0] << ": error, refusing to run as root" << endl;
		return 1;
	}

	// parse options
	{
	int i;
		for (i = 1; i < argc && argv[i][0] == '-'; ++i)
			switch (argv[i][1]) {
				case 'V' :
					cout << VERSION << endl;
					return 0;
				break;
				case 'v' : // verbose
					verbose = true;
				break;
				case 't' : // tcsh
					shell = TCSH;
				break;
				default :
					cerr << argv[0] << ": error, unknown option '" << argv[i] << "'" << endl;
				case 'h' :
					cout << "usage: " << argv[0] << " [-v] [-t] [-h] <old directory>" << endl
						<< "  -v    verbose" << endl
						<< "  -t    display TCSH commands rather than BASH" << endl
						<< "  -h    help" << endl
						<< "  -V    show version" << endl;
					return 0;
				break;
			}
		if (i >= argc) {
			cerr << argv[0] << ": error, path must be specified" << endl;
			return 1;
		}
		olddir = argv[i];
	}

	// get base working directories
vector<string> last_dir, current_dir;

	{
	char tmp[strlen(olddir) + 1];
		strcpy(tmp, olddir);
		if (tmp[0] != '/') {
			cerr << argv[0] << ": error, path must be absolute" << endl;
			return 1;
		}
		while (tmp[strlen(tmp) - 1] == '/') tmp[strlen(tmp) - 1] = 0;
		last_dir = split(tmp);
	}
	{
	char tmp[PATH_MAX];
		getcwd(tmp, PATH_MAX);
		while (tmp[strlen(tmp) - 1] == '/') tmp[strlen(tmp) - 1] = 0;
		current_dir = split(tmp);
	}

unsigned common_root;

	// Find common root path
	for (common_root = 0; last_dir.size() > common_root && current_dir.size() > common_root && last_dir[common_root] == current_dir[common_root] ; common_root++) ;
	if (common_root) common_root--;

	// Move up to common root from old path
	for (unsigned i = last_dir.size() - 1; i > common_root; i--) {
	string leaving = join(last_dir, i);
	string filename = leaving + "/.onleave";


		if (!readable(filename))
			if (readable(leaving + "/.onexit"))
				filename = leaving + "/.onexit";

		if (readable(filename)) {
			if (!goodpermissions(leaving + "/.onleave"))
				cerr << argv[0] << ": warning, not executing \"" << leaving << "/.onleave\" as it has bad permissions/ownership" << endl;
			else {
				if (verbose) cerr << "ondir: called .onleave in '" << leaving << "'" << endl;
				if (shell == TCSH) 
					cout << "setenv ONDIRWD \"" << leaving << "\";" << endl;
				else
					cout << "export ONDIRWD=\"" << leaving << "\";" << endl;
				if (shell == BASH)
					cout << ".";
				if (shell == TCSH)
					cout << "source";
				cout << " \"" << leaving << "/.onleave\";" << endl;
			}
			if (shell == TCSH) 
				cout << endl << "unsetenv ONDIRWD" << endl;
			else
				cout << endl << "unset ONDIRWD" << endl;
		} else 
		if (rc.leavePath(leaving) != "") {
		string code = rc.leavePath(leaving);

			if (verbose) cerr << "ondir: executing leave code for '" << leaving << "'" << endl;
			if (shell == TCSH) 
				cout << "setenv ONDIRWD \"" << leaving << "\";" << endl;
			else
				cout << "export ONDIRWD=\"" << leaving << "\";" << endl;
			cout << str::replace("\n", ";\n", str::trim(code)) << ";" << endl;
			if (shell == TCSH)
				cout << endl << "unsetenv ONDIRWD;" << endl;
			else
				cout << endl << "unset ONDIRWD" << endl;
		}
	}
	// Move in to new path from common root
	for (unsigned i = common_root + 1; i < current_dir.size(); i++) {
	string entering = join(current_dir, i);

		if (readable(entering + "/.onenter")) {
			if (!goodpermissions(entering + "/.onenter"))
				cerr << argv[0] << ": warning, not executing \"" << entering << "/.onenter\" as it has bad permissions/ownership" << endl;
			else {
				if (verbose) cerr << "ondir: called .onenter in '" << entering << "'" << endl;
				if (shell == TCSH) 
					cout << "setenv ONDIRWD \"" << entering << "\";" << endl;
				else
					cout << "export ONDIRWD=\"" << entering << "\";" << endl;
				if (shell == BASH)
					cout << ".";
				if (shell == TCSH)
					cout << "source";
				cout << " \"" << entering << "/.onenter\";" << endl;
			}
			if (shell == TCSH) 
				cout << endl << "unsetenv ONDIRWD" << endl;
			else
				cout << endl << "unset ONDIRWD" << endl;
		} else 
		if (rc.enterPath(entering) != "") {
		string code = rc.enterPath(entering);

			if (verbose) cerr << "ondir: executing enter code for '" << entering << "'" << endl;
			if (shell == TCSH) 
				cout << "setenv ONDIRWD \"" << entering << "\";" << endl;
			else
				cout << "export ONDIRWD=\"" << entering << "\";" << endl;
			cout << str::replace("\n", ";\n", str::trim(code)) << ";" << endl;
			if (shell == TCSH) 
				cout << endl << "unsetenv ONDIRWD" << endl;
			else
				cout << endl << "unset ONDIRWD" << endl;
		}
	}
	return 0;
}

/* Split a path into a vector */
vector<string> split(char const *str) {
vector<string> out;
char const *last;

	if (str[0] == '/') str++;
	for (last = str; *str; str++) {
		if (*str == '/') {
			out.push_back(string(last, str - last));
			last = str + 1;
		}
	}
	out.push_back(string(last, str - last));
	return out;
}

/* Join a split path into a string */
string join(vector<string> const &in, unsigned to) {
string out;

	for (unsigned i = 0; i <= to; ++i)
		out += "/" + in[i];
	return out;
}

/* Return true if file is readable */
bool readable(string const &file) {
struct stat s;

	if (!stat(file.c_str(), &s) && 
		s.st_mode & (S_IRUSR | S_IRGRP | S_IROTH))
			return true;
	return false;
}

/* Return true if file has good permissions */
bool goodpermissions(string const &file) {
struct stat s;

	if (!stat(file.c_str(), &s) && s.st_uid == getuid() &&
		!(s.st_mode & (S_IWGRP | S_IWOTH)))
			return true;
	return false;
}

void enterPath(string const &path) {
}

void exitPath(string const &path) {
}

string expandString(string const &str) {
string out;

	for (char const *i = str.c_str(); *i != 0; ++i)
		if (*i == '$') {
		string word;

			++i;
			while (isalnum(*i) || *i == '_') {
				word += *i;
				++i;
			}
			--i;
			if (getenv(word.c_str()))
				out += getenv(word.c_str());
		} else
			out += *i;
	return out;
}
